@@ -1,4 +1,5 @@
 __author__ = "Akshay Mane"
+__author__ = "Mathieu Millet"
 
 import json
 import os
@@ -8,7 +9,7 @@ from functools import wraps
 import requests
 import urllib3
 import logging
-from typing import List
+from typing import List, Any
 from os.path import join, normpath
 
 # Disable insecure connections warnings
@@ -126,7 +127,7 @@ class FortiManager:
                     ],
                 "session": self.sessionid
             }
-        get_adoms = session.post(url=self.base_url, json=payload, verify=False)
+        get_adoms = session.post(url=self.base_url, json=payload, verify=self.verify)
         return get_adoms.json()["result"]
 
     def get_devices(self):
@@ -361,6 +362,30 @@ class FortiManager:
             url=self.base_url, json=payload, verify=self.verify)
         return get_address_objects.json()["result"]
 
+    # Firewall Object v6 Methods
+    def get_firewall_address_v6_objects(self, name=False):
+        """
+        Get all the address v6 objects data stored in FortiManager
+        :return: Response of status code with data in JSON Format
+        """
+        url = f"pm/config/adom/{self.adom}/obj/firewall/address6"
+        if name:
+            url = f"pm/config/adom/{self.adom}/obj/firewall/address6/{name}"
+        session = self.login()
+        payload = \
+            {
+                "method": "get",
+                "params": [
+                    {
+                        "url": url
+                    }
+                ],
+                "session": self.sessionid
+            }
+        get_address_objects = session.post(
+            url=self.base_url, json=payload, verify=self.verify)
+        return get_address_objects.json()["result"]
+
     def add_firewall_address_object(self, name, subnet: list, associated_interface="any", object_type=0,
                                     allow_routing=0):
         """
@@ -387,6 +412,28 @@ class FortiManager:
         add_address_object = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return add_address_object.json()["result"]
+    
+    def add_firewall_address_v6_object(self, name, subnet6: str, object_type=0):
+        """
+        Create an address object using provided info
+        :param name: Enter object name that is to be created        
+        :param subnet: Enter the subnet in a string format "200x:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx/"128"
+        :param object_type:
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        payload = {
+            "method": "add",
+            "params": [{"data": {                                
+                "name": name,
+                "ip6": subnet6,
+                "type": object_type},
+                "url": f"pm/config/adom/{self.adom}/obj/firewall/address6"}],
+            "session": self.sessionid}
+
+        add_address_object = session.post(
+            url=self.base_url, json=payload, verify=self.verify)
+        return add_address_object.json()["result"]
 
     def add_dynamic_object(self, name, device, subnet=list, comment=None):
         """
@@ -463,6 +510,31 @@ class FortiManager:
             url=self.base_url, data=payload, verify=self.verify)
         return update_firewall_object.json()["result"]
 
+    def update_firewall_address_v6_object(self, name, **data):
+        """
+        Get the name of the address object and update it with your data
+        :param name: Enter the name of the object that needs to be updated
+        :param data: You can get the **kwargs parameters with "show_params_for_object_v6_update()" method
+        :return: Response of status code with data in JSON Format
+        """
+        data = self.make_data(_for="object", **data)
+        session = self.login()
+        payload = \
+            {
+                "method": "update",
+                "params": [
+                    {
+                        "data": data,
+                        "url": f"pm/config/adom/{self.adom}/obj/firewall/address6/{name}"
+                    }
+                ],
+                "session": self.sessionid
+            }
+        payload = repr(payload)
+        update_firewall_object = session.post(
+            url=self.base_url, data=payload, verify=self.verify)
+        return update_firewall_object.json()["result"]
+
     def delete_firewall_address_object(self, object_name):
         """
         Delete the address object if no longer needed using object name
@@ -484,6 +556,27 @@ class FortiManager:
             url=self.base_url, json=payload, verify=self.verify)
         return delete_address_object.json()["result"]
 
+    def delete_firewall_address_v6_object(self, object_name):
+        """
+        Delete the address object if no longer needed using object name
+        :param object_name: Enter the Object name you want to delete
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        payload = \
+            {
+                "method": "delete",
+                "params": [
+                    {
+                        "url": f"pm/config/adom/{self.adom}/obj/firewall/address6/{object_name}"
+                    }
+                ],
+                "session": self.sessionid
+            }
+        delete_address_object = session.post(
+            url=self.base_url, json=payload, verify=self.verify)
+        return delete_address_object.json()["result"]
+
     # Firewall Address Groups Methods
     def get_address_groups(self, name=False):
         """
@@ -508,7 +601,31 @@ class FortiManager:
         get_address_group = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return get_address_group.json()["result"]
-
+    
+    def get_address_v6_groups(self, name=False):
+        """
+        Get the address groups created in your FortiManager
+        :param name: You can filter out the specific address group which you want to see
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        url = f"pm/config/adom/{self.adom}/obj/firewall/addrgrp6"
+        if name:
+            url = f"pm/config/adom/{self.adom}/obj/firewall/addrgrp6/{name}"
+        payload = \
+            {
+                "method": "get",
+                "params": [
+                    {
+                        "url": url
+                    }
+                ],
+                "session": self.sessionid
+            }
+        get_address_group = session.post(
+            url=self.base_url, json=payload, verify=self.verify)
+        return get_address_group.json()["result"]
+    
     def add_address_group(self, name, members=list):
         """
         Create your own group with just 2 parameters
@@ -534,7 +651,33 @@ class FortiManager:
         add_address_group = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return add_address_group.json()["result"]
-
+    
+    def add_address_v6_group(self, name, members=list):
+        """
+        Create your own group with just 2 parameters
+        :param name: Enter the name of the address group                eg."Test_Group"
+        :param members: pass your object names as members in a list     eg. ["LAN_10.1.1.0_24, "INTERNET"]
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        payload = \
+            {
+                "method": "add",
+                "params": [
+                    {
+                        "data": {
+                            "name": name,
+                            "member": members,
+                        },
+                        "url": f"pm/config/adom/{self.adom}/obj/firewall/addrgrp6"
+                    }
+                ],
+                "session": self.sessionid
+            }
+        add_address_group = session.post(
+            url=self.base_url, json=payload, verify=self.verify)
+        return add_address_group.json()["result"]
+    
     def update_address_group(self, name, object_name, do="add"):
         """
         Update Members of the Address group
@@ -570,6 +713,41 @@ class FortiManager:
             url=self.base_url, json=payload, verify=False)
         return update_address_group.json()["result"]
 
+    def update_address_v6_group(self, name, object_name, do="add"):
+        """
+        Update Members of the Address group
+        :param name: Specify the name of the Address group you want to update
+        :param object_name: Specify name of the object you wish to update(add/remove) in Members List
+        :param do: Specify if you want to add or remove the object from the members list
+                    do="add"    will add the object in the address group
+                    do="remove" will remove the object from address group
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        get_addr_v6_group = self.get_address_v6_groups(name=name)
+        members = get_addr_v6_group[0]['data']['member']
+        if do == "add":
+            members.append(object_name)
+        elif do == "remove":
+            members.remove(object_name)
+
+        payload = \
+            {
+                "method": "update",
+                "params": [
+                    {
+                        "data": {
+                            "member": members,
+                        },
+                        "url": f"pm/config/adom/{self.adom}/obj/firewall/addrgrp6/{name}"
+                    }
+                ],
+                "session": self.sessionid
+            }
+        update_address_group = session.post(
+            url=self.base_url, json=payload, verify=False)
+        return update_address_group.json()["result"]
+
     def delete_address_group(self, name):
         """
         Delete the Address group if no longer needed
@@ -593,6 +771,29 @@ class FortiManager:
             url=self.base_url, json=payload, verify=False)
         return delete_address_group.json()["result"]
 
+    def delete_address_v6_group(self, name):
+        """
+        Delete the Address group if no longer needed
+        :param name: Specify the name of the address you wish to delete
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        payload = \
+            {
+                "method": "delete",
+                "params": [
+                    {
+                        "data": {
+                        },
+                        "url": f"pm/config/adom/{self.adom}/obj/firewall/addrgrp6/{name}"
+                    }
+                ],
+                "session": self.sessionid
+            }
+        delete_address_group = session.post(
+            url=self.base_url, json=payload, verify=False)
+        return delete_address_group.json()["result"]
+
     # Firewall Virtual IP objects
     def get_firewall_vip_objects(self, name=False):
         """
@@ -617,6 +818,47 @@ class FortiManager:
             url=self.base_url, json=payload, verify=self.verify)
         return get_vip_objects.json()["result"]
 
+    # Header
+    def get_global_header_policies(self, policy_package_name="default", policyid=False):
+        """
+        Get global header policies
+        """
+        url = f"pm/config/global/pkg/{policy_package_name}/global/header/policy"
+        if policyid:
+            url = url + str(policyid)
+        session = self.login()
+        payload = {
+            "method": "get",
+            "params": [
+                {
+                    "url": url
+                }
+            ],
+            "session": self.sessionid
+        }
+        get_global_header_policies = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return get_global_header_policies.json()["result"]
+
+    def get_firewall_header_policies(self, policy_package_name="default", policyid=False):
+        """
+        Get adom header policies
+        """
+        url = f"pm/config/adom/{self.adom}/obj/global/header/policy"
+        if policyid:
+            url = url + str(policyid)
+        session = self.login()
+        payload = {
+            "method": "get",
+            "params": [
+                {
+                    "url": url
+                }
+            ],
+            "session": self.sessionid
+        }
+        get_firewall_header_policies = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return get_firewall_header_policies.json()["result"]
+
     # Firewall Interfaces
     def get_device_interfaces(self, device):
         """
@@ -687,6 +929,23 @@ class FortiManager:
         get_firewall_policies = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return get_firewall_policies.json()["result"]
+	
+    def get_dhcp(self, device):
+        """
+        Get dhcp details from the devices.
+        :param device: Specify name of the device.
+        """
+        session = self.login()
+        payload = \
+            {
+                "method": "exec",
+                "params": [
+                    {"url": "sys/proxy/json",
+                     "data": {"target": [f"adom/{self.adom}/device/{device}"], "action": "get",
+                              "resource": "/api/v2/monitor/system/dhcp/select?&vdom=root&ipv6=true&scope=global"}}]}
+        payload.update(session=self.sessionid)
+        get_interfaces = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return get_interfaces.json()["result"] 
 
     def add_firewall_policy(self, policy_package_name: str, name: str, source_interface: str,
                             source_address: str, destination_interface: str, destination_address: str,
@@ -735,6 +994,57 @@ class FortiManager:
             url=self.base_url, json=payload, verify=self.verify)
         return add_policy.json()
 
+    def add_firewall_policy_with_v6(self, policy_package_name: str, name: str, source_interface: str,
+                            source_address: Any, source_address6: Any, destination_interface: str, destination_address: Any, destination_address6: Any,
+                            service: str, nat='disable', schedule="always", action=1, logtraffic=2):
+        """
+        Create your own policy in FortiManager using the instance parameters.
+        :param policy_package_name: Enter the name of the policy package                eg. "default"
+        :param name: Enter the policy name in a string format                           eg. "Test Policy"
+        :param source_interface: Enter the source interface in a string format          eg. "port1"
+        :param source_address: Enter the src. address object name in string format or in a list Format      eg. "LAN_10.1.1.0_24" or ["LAN_10.1.1.0_24", "LAN_10.2.2.0_24"]
+        :param source_address6: Enter the src. address v6 object name in string format or in a list Format      eg. "LAN_200x-000-" or ["LAN_2001-000", "LAN_2002-1000"]
+        :param destination_interface: Enter the source interface in a string format     eg. "port2"
+        :param destination_address: Enter the dst. address object name in string or list format                  eg. "WAN_100.25.1.63_32" or ["WAN1", "WAN2"]
+        :param destination_address6: Enter the dst. address object name in string or list format                  eg. "WANv6_200a-200a-" or or ["WAN1v6", "WAN2v6"]
+        :param service: Enter the service you want to permit or deny in string          eg. "ALL_TCP"
+        :param nat: Enter enable or disable for nat in a string format                  eg. 'disable'
+        :param schedule: Schedule time is kept 'always' as default.
+        :param action: Permit(1) or Deny(0) the traffic. Default is set to Permit.
+        :param logtraffic: Specify if you need to log all traffic or specific in int format.
+                            logtraffic=0: Means No Log
+                            logtraffic=1 Means Log Security Events
+                            logtraffic=2 Means Log All Sessions
+        :return: Response of status code with data in JSON Format
+        """
+        session = self.login()
+        payload = {
+            "method": "add",
+            "params": [
+                {
+                    "data": {
+                        "dstaddr": destination_address,
+                        "dstaddr6" : destination_address6,
+                        "dstintf": destination_interface,
+                        "logtraffic": logtraffic,
+                        "name": name,
+                        "schedule": schedule,
+                        "service": service,
+                        "srcaddr": source_address,
+                        "srcaddr6" : source_address6,
+                        "srcintf": source_interface,
+                        "action": action,
+                        "nat": nat
+                    },
+                    "url": f"pm/config/adom/{self.adom}/pkg/{policy_package_name}/firewall/policy/"
+                }
+            ],
+            "session": self.sessionid
+        }
+        add_policy = session.post(
+            url=self.base_url, json=payload, verify=self.verify)
+        return add_policy.json()
+
     def update_firewall_policy(self, policy_package_name, policyid, **data):
         """
         Update your policy with your specific needs
@@ -887,6 +1197,20 @@ class FortiManager:
         """
         return docs
 
+    @staticmethod
+    def show_params_for_object_v6_update():
+        docs = \
+            """
+        Parameters to create/update address object:
+
+        PARAMETERS                   FIREWALL OBJECT SETTINGS
+        comment(str)                : Comments
+        object_name(str)            : Address Name
+        subnet(str)                 : IP/Netmask
+        object_type(int)            : Type
+        """
+        return docs
+
     @staticmethod
     def show_params_for_policy_update():
         docs = \
@@ -907,6 +1231,28 @@ class FortiManager:
         """
         return docs
 
+    @staticmethod
+    def show_params_for_policy_v6_update():
+        docs = \
+            """
+        Parameters to create/update Policy with v6 address objects:
+
+        PARAMETERS                       FIREWALL POLICY SETTINGS
+        name(str)                       : Name
+        source_interface(str)           : Incoming Interface
+        source_address(str|list)        : Source Address Object or List of Address Object
+        source_address6(str|list)       : Source Address v6 or List
+        destination_interface(str)      : Destination Interface
+        destination_address(str|list)   : Destination Address or List of Address Object
+        destination_address6(str|list)  : Destination Address v6 or List
+        service(str)                    : Service
+        schedule(str)                   : Schedule
+        action(int)                     : Action
+        logtraffic(int)                 : Log Traffic
+        comment(str)                    : Comments
+        """
+        return docs
+
     def custom_api(self, payload):
         """
         Execute an API call manually by defining the payload
